@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: arguments.c 101 2011-01-17 03:09:05Z schmidtw $
+ * $Id: arguments.c 149 2013-02-21 22:54:47Z slarge $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,64 +56,102 @@ struct target_mapping_structure {
  * space needed by the bootloader.  The size of the bootloader is set
  * by BOOTSZ0/BOOTSZ1 fuse bits; here we assume the bootloader is 4kb or 8kb.
  * The window used for the bootloader is at the top of the of memory.
+ *
+ * For at89c5130/1 the bootloader is outside the normal flash area.
+ * Which is why the boot size is marked as 0 bytes.
+ *
+ * VID and PID are the USB identifiers returned by the DFU bootloader.
+ * They are defined by Atmel's bootloader code, and are not in the chip datasheet.
+ * An incomplete list can be found the the various DFU bootloader docs.
+ * If you plug the device in, lsusb or the Windows device manager can tell you
+ * the VID and PID values.
  */
 
 /* ----- target specific structures ----------------------------------------- */
 static struct target_mapping_structure target_map[] = {
-    { "at89c51snd1c",   tar_at89c51snd1c,   adc_8051,  0x2FFF, 0x03eb, 0x10000, 0x1000, true,  128, false, true,  0,   0      },
-    { "at89c51snd2c",   tar_at89c51snd2c,   adc_8051,  0x2FFF, 0x03eb, 0x10000, 0x1000, true,  128, false, true,  0,   0      },
-    { "at89c5130",      tar_at89c5130,      adc_8051,  0x2FFD, 0x03eb, 0x04000, 0x0000, true,  128, false, true,  128, 0x0400 },    /* The bootloader is out of the normal flash. */
-    { "at89c5131",      tar_at89c5131,      adc_8051,  0x2FFD, 0x03eb, 0x08000, 0x0000, true,  128, false, true,  128, 0x0400 },    /* The bootloader is out of the normal flash. */
-    { "at89c5132",      tar_at89c5132,      adc_8051,  0x2FFF, 0x03eb, 0x10000, 0x0C00, true,  128, false, true,  0,   0      },
-    { "at90usb1287",    tar_at90usb1287,    adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x2000, true,  128, true,  false, 128, 0x1000 },
-    { "at90usb1286",    tar_at90usb1286,    adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x2000, true,  128, true,  false, 128, 0x1000 },
-    { "at90usb1287-4k", tar_at90usb1287_4k, adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x1000, true,  128, true,  false, 128, 0x1000 },
-    { "at90usb1286-4k", tar_at90usb1286_4k, adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x1000, true,  128, true,  false, 128, 0x1000 },
-    { "at90usb647",     tar_at90usb647,     adc_AVR,   0x2FF9, 0x03eb, 0x10000, 0x2000, true,  128, true,  false, 128, 0x0800 },
-    { "at90usb646",     tar_at90usb646,     adc_AVR,   0x2FF9, 0x03eb, 0x10000, 0x2000, true,  128, true,  false, 128, 0x0800 },
-    { "at90usb162",     tar_at90usb162,     adc_AVR,   0x2FFA, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x0200 },
-    { "at90usb82",      tar_at90usb82,      adc_AVR,   0x2FF7, 0x03eb, 0x02000, 0x1000, true,  128, true,  false, 128, 0x0200 },
-    { "atmega32u6",     tar_atmega32u6,     adc_AVR,   0x2FF2, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x0400 },
-    { "atmega32u4",     tar_atmega32u4,     adc_AVR,   0x2FF4, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x0400 },
-    { "atmega32u2",     tar_atmega32u2,     adc_AVR,   0x2FF0, 0x03eb, 0x08000, 0x1000, true,  128, true, false,  128, 0x0400 },
-    { "atmega16u4",     tar_atmega16u4,     adc_AVR,   0x2FF3, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x0200 },
-    { "atmega8u2",      tar_atmega8u2,      adc_AVR,   0x2FEE, 0x03eb, 0x02000, 0x1000, true,  128, true,  false, 128, 0x0200 },
-    { "at32uc3a0128",   tar_at32uc3a0128,   adc_AVR32, 0x2FF8, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a1128",   tar_at32uc3a1128,   adc_AVR32, 0x2FF8, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a0256",   tar_at32uc3a0256,   adc_AVR32, 0x2FF8, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a1256",   tar_at32uc3a1256,   adc_AVR32, 0x2FF8, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a0512",   tar_at32uc3a0512,   adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a1512",   tar_at32uc3a1512,   adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a0512es", tar_at32uc3a0512es, adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a1512es", tar_at32uc3a1512es, adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a364",    tar_at32uc3a364,    adc_AVR32, 0x2FF1, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a364s",   tar_at32uc3a364s,   adc_AVR32, 0x2FF1, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a3128",   tar_at32uc3a3128,   adc_AVR32, 0x2FF1, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a3128s",  tar_at32uc3a3128s,  adc_AVR32, 0x2FF1, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a3256",   tar_at32uc3a3256,   adc_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3a3256s",  tar_at32uc3a3256s,  adc_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b064",    tar_at32uc3b064,    adc_AVR32, 0x2FF6, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b164",    tar_at32uc3b164,    adc_AVR32, 0x2FF6, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b0128",   tar_at32uc3b0128,   adc_AVR32, 0x2FF6, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b1128",   tar_at32uc3b1128,   adc_AVR32, 0x2FF6, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b0256",   tar_at32uc3b0256,   adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b1256",   tar_at32uc3b1256,   adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b0256es", tar_at32uc3b0256es, adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b1256es", tar_at32uc3b1256es, adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b0512",   tar_at32uc3b0512,   adc_AVR32, 0x2FF6, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3b1512",   tar_at32uc3b1512,   adc_AVR32, 0x2FF6, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c064",    tar_at32uc3c064,    adc_AVR32, 0x2FEB, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c0128",   tar_at32uc3c0128,   adc_AVR32, 0x2FEB, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c0256",   tar_at32uc3c0256,   adc_AVR32, 0x2FEB, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c0512",   tar_at32uc3c0512,   adc_AVR32, 0x2FEB, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c164",    tar_at32uc3c164,    adc_AVR32, 0x2FEB, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c1128",   tar_at32uc3c1128,   adc_AVR32, 0x2FEB, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c1256",   tar_at32uc3c1256,   adc_AVR32, 0x2FEB, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c1512",   tar_at32uc3c1512,   adc_AVR32, 0x2FEB, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c264",    tar_at32uc3c264,    adc_AVR32, 0x2FEB, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c2128",   tar_at32uc3c2128,   adc_AVR32, 0x2FEB, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c2256",   tar_at32uc3c2256,   adc_AVR32, 0x2FEB, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
-    { "at32uc3c2512",   tar_at32uc3c2512,   adc_AVR32, 0x2FEB, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "at89c51snd1c",   tar_at89c51snd1c,   ADC_8051,  0x2FFF, 0x03eb, 0x10000, 0x1000, true,  128, false, true,  0,   0      },
+    { "at89c51snd2c",   tar_at89c51snd2c,   ADC_8051,  0x2FFF, 0x03eb, 0x10000, 0x1000, true,  128, false, true,  0,   0      },
+    { "at89c5130",      tar_at89c5130,      ADC_8051,  0x2FFD, 0x03eb, 0x04000, 0x0000, true,  128, false, true,  128, 0x0400 },
+    { "at89c5131",      tar_at89c5131,      ADC_8051,  0x2FFD, 0x03eb, 0x08000, 0x0000, true,  128, false, true,  128, 0x0400 },
+    { "at89c5132",      tar_at89c5132,      ADC_8051,  0x2FFF, 0x03eb, 0x10000, 0x0C00, true,  128, false, true,  0,   0      },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "at90usb1287",    tar_at90usb1287,    ADC_AVR,   0x2FFB, 0x03eb, 0x20000, 0x2000, true,  128, true,  false, 128, 0x1000 },
+    { "at90usb1286",    tar_at90usb1286,    ADC_AVR,   0x2FFB, 0x03eb, 0x20000, 0x2000, true,  128, true,  false, 128, 0x1000 },
+    { "at90usb1287-4k", tar_at90usb1287_4k, ADC_AVR,   0x2FFB, 0x03eb, 0x20000, 0x1000, true,  128, true,  false, 128, 0x1000 },
+    { "at90usb1286-4k", tar_at90usb1286_4k, ADC_AVR,   0x2FFB, 0x03eb, 0x20000, 0x1000, true,  128, true,  false, 128, 0x1000 },
+    { "at90usb647",     tar_at90usb647,     ADC_AVR,   0x2FF9, 0x03eb, 0x10000, 0x2000, true,  128, true,  false, 128, 0x0800 },
+    { "at90usb646",     tar_at90usb646,     ADC_AVR,   0x2FF9, 0x03eb, 0x10000, 0x2000, true,  128, true,  false, 128, 0x0800 },
+    { "at90usb162",     tar_at90usb162,     ADC_AVR,   0x2FFA, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x0200 },
+    { "at90usb82",      tar_at90usb82,      ADC_AVR,   0x2FF7, 0x03eb, 0x02000, 0x1000, true,  128, true,  false, 128, 0x0200 },
+    { "atmega32u6",     tar_atmega32u6,     ADC_AVR,   0x2FF2, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x0400 },
+    { "atmega32u4",     tar_atmega32u4,     ADC_AVR,   0x2FF4, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x0400 },
+    { "atmega32u2",     tar_atmega32u2,     ADC_AVR,   0x2FF0, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x0400 },
+    { "atmega16u4",     tar_atmega16u4,     ADC_AVR,   0x2FF3, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x0200 },
+    { "atmega16u2",     tar_atmega16u2,     ADC_AVR,   0x2FEF, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x0200 },
+    { "atmega8u2",      tar_atmega8u2,      ADC_AVR,   0x2FEE, 0x03eb, 0x02000, 0x1000, true,  128, true,  false, 128, 0x0200 },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "at32uc3a0128",   tar_at32uc3a0128,   ADC_AVR32, 0x2FF8, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a1128",   tar_at32uc3a1128,   ADC_AVR32, 0x2FF8, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a0256",   tar_at32uc3a0256,   ADC_AVR32, 0x2FF8, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a1256",   tar_at32uc3a1256,   ADC_AVR32, 0x2FF8, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a0512",   tar_at32uc3a0512,   ADC_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a1512",   tar_at32uc3a1512,   ADC_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a0512es", tar_at32uc3a0512es, ADC_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a1512es", tar_at32uc3a1512es, ADC_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a364",    tar_at32uc3a364,    ADC_AVR32, 0x2FF1, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a364s",   tar_at32uc3a364s,   ADC_AVR32, 0x2FF1, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a3128",   tar_at32uc3a3128,   ADC_AVR32, 0x2FF1, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a3128s",  tar_at32uc3a3128s,  ADC_AVR32, 0x2FF1, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a3256",   tar_at32uc3a3256,   ADC_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a3256s",  tar_at32uc3a3256s,  ADC_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3a4256s",  tar_at32uc3a4256s,  ADC_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "at32uc3b064",    tar_at32uc3b064,    ADC_AVR32, 0x2FF6, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b164",    tar_at32uc3b164,    ADC_AVR32, 0x2FF6, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b0128",   tar_at32uc3b0128,   ADC_AVR32, 0x2FF6, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b1128",   tar_at32uc3b1128,   ADC_AVR32, 0x2FF6, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b0256",   tar_at32uc3b0256,   ADC_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b1256",   tar_at32uc3b1256,   ADC_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b0256es", tar_at32uc3b0256es, ADC_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b1256es", tar_at32uc3b1256es, ADC_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b0512",   tar_at32uc3b0512,   ADC_AVR32, 0x2FF6, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3b1512",   tar_at32uc3b1512,   ADC_AVR32, 0x2FF6, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "at32uc3c064",    tar_at32uc3c064,    ADC_AVR32, 0x2FEB, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c0128",   tar_at32uc3c0128,   ADC_AVR32, 0x2FEB, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c0256",   tar_at32uc3c0256,   ADC_AVR32, 0x2FEB, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c0512",   tar_at32uc3c0512,   ADC_AVR32, 0x2FEB, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c164",    tar_at32uc3c164,    ADC_AVR32, 0x2FEB, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c1128",   tar_at32uc3c1128,   ADC_AVR32, 0x2FEB, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c1256",   tar_at32uc3c1256,   ADC_AVR32, 0x2FEB, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c1512",   tar_at32uc3c1512,   ADC_AVR32, 0x2FEB, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c264",    tar_at32uc3c264,    ADC_AVR32, 0x2FEB, 0x03eb, 0x10000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c2128",   tar_at32uc3c2128,   ADC_AVR32, 0x2FEB, 0x03eb, 0x20000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c2256",   tar_at32uc3c2256,   ADC_AVR32, 0x2FEB, 0x03eb, 0x40000, 0x2000, false, 512, false, true,  0,   0      },
+    { "at32uc3c2512",   tar_at32uc3c2512,   ADC_AVR32, 0x2FEB, 0x03eb, 0x80000, 0x2000, false, 512, false, true,  0,   0      },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "atxmega64a1u",   tar_atxmega64a1u,   ADC_XMEGA, 0x2FE8, 0x03eb, 0x10000, 0x1000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega128a1u",  tar_atxmega128a1u,  ADC_XMEGA, 0x2FED, 0x03eb, 0x20000, 0x2000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega64a3u",   tar_atxmega64a3u,   ADC_XMEGA, 0x2FE5, 0x03eb, 0x10000, 0x1000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega128a3u",  tar_atxmega128a3u,  ADC_XMEGA, 0x2FE6, 0x03eb, 0x20000, 0x2000, true,  512, true,  false, 32,  0x0800 },
+    { "atxmega192a3u",  tar_atxmega192a3u,  ADC_XMEGA, 0x2FE7, 0x03eb, 0x30000, 0x2000, true,  512, true,  false, 32,  0x0800 },
+    { "atxmega256a3u",  tar_atxmega256a3u,  ADC_XMEGA, 0x2FEC, 0x03eb, 0x40000, 0x2000, true,  512, true,  false, 32,  0x1000 },
+    { "atxmega16a4u",   tar_atxmega16a4u,   ADC_XMEGA, 0x2FE3, 0x03eb, 0x04000, 0x1000, true,  256, true,  false, 32,  0x0400 },
+    { "atxmega32a4u",   tar_atxmega32a4u,   ADC_XMEGA, 0x2FE4, 0x03eb, 0x08000, 0x1000, true,  256, true,  false, 32,  0x0400 },
+    { "atxmega64a4u",   tar_atxmega64a4u,   ADC_XMEGA, 0x2FDD, 0x03eb, 0x10000, 0x1000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega128a4u",  tar_atxmega128a4u,  ADC_XMEGA, 0x2FDE, 0x03eb, 0x20000, 0x2000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega256a3bu", tar_atxmega256a3bu, ADC_XMEGA, 0x2FE2, 0x03eb, 0x40000, 0x2000, true,  512, true,  false, 32,  0x1000 },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "atxmega64b1",    tar_atxmega64b1,    ADC_XMEGA, 0x2FE1, 0x03eb, 0x10000, 0x1000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega128b1",   tar_atxmega128b1,   ADC_XMEGA, 0x2FEA, 0x03eb, 0x20000, 0x2000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega64b3",    tar_atxmega64b3,    ADC_XMEGA, 0x2FDF, 0x03eb, 0x10000, 0x1000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega128b3",   tar_atxmega128b3,   ADC_XMEGA, 0x2FE0, 0x03eb, 0x20000, 0x2000, true,  256, true,  false, 32,  0x0800 },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootHi FPage Abort IF     EPage ESize
+    { "atxmega64c3",    tar_atxmega64c3,    ADC_XMEGA, 0x2FD6, 0x03eb, 0x10000, 0x1000, true,  256, true,  false, 32,  0x0800 },
+    { "atxmega128c3",   tar_atxmega128c3,   ADC_XMEGA, 0x2FD7, 0x03eb, 0x20000, 0x2000, true,  512, true,  false, 32,  0x0800 },
+    { "atxmega256c3",   tar_atxmega256c3,   ADC_XMEGA, 0x2FDA, 0x03eb, 0x40000, 0x2000, true,  512, true,  false, 32,  0x1000 },
+    { "atxmega384c3",   tar_atxmega384c3,   ADC_XMEGA, 0x2FDB, 0x03eb, 0x60000, 0x2000, true,  512, true,  false, 32,  0x1000 },
     { NULL }
 };
 
@@ -130,9 +168,9 @@ static struct option_mapping_structure command_map[] = {
     { "get",          com_get       },
     { "getfuse",      com_getfuse   },
     { "setfuse",      com_setfuse   },
+    { "setsecure",    com_setsecure },
     { "reset",        com_reset     },
     { "start",        com_start_app },
-    { "version",      com_version   },
     { NULL }
 };
 
@@ -191,54 +229,71 @@ static struct option_mapping_structure setfuse_map[] = {
     { NULL }
 };
 
-
-static void usage()
+static void list_targets()
 {
     struct target_mapping_structure *map = NULL;
+    int col = 0;
 
     map = target_map;
 
-    fprintf( stderr, PACKAGE_STRING "\n");
-    fprintf( stderr, "Usage: dfu-programmer target command [command-options] "
-                     "[global-options] [file|data]\n" );
     fprintf( stderr, "targets:\n" );
     while( 0 != *((int32_t *) map) ) {
-        fprintf( stderr, "        %s\n", map->name );
+        if( 0 == col ) {
+            fprintf( stderr, " " );
+        }
+        fprintf( stderr, "   %-16s", map->name );
+        if( 4 == ++col ) {
+            fprintf( stderr, "\n" );
+            col = 0;
+        }
         map++;
     }
-    fprintf( stderr, "global-options: --quiet, --debug level\n" );
+    if( 0 != col )
+        fprintf( stderr, "\n" );
+}
+
+static void basic_help()
+{
+    fprintf( stderr, "Type 'dfu-programmer --help'    for a list of commands\n" );
+    fprintf( stderr, "     'dfu-programmer --targets' to list supported target devices\n" );
+    fprintf( stderr, "     'dfu-programmer --version' to show version information\n" );
+}
+
+static void usage()
+{
+    fprintf( stderr, "Usage: dfu-programmer target[:usb-bus,usb-addr] command [options] "
+                     "[global-options] [file|data]\n\n" );
+
+    fprintf( stderr, "global-options:\n"
+                     "        --quiet\n"
+                     "        --debug level    (level is an integer specifying level of detail)\n"
+                     "        Global options can be used with any command and must come\n"
+                     "        after the command and before any file or data value\n\n" );
     fprintf( stderr, "commands:\n" );
     fprintf( stderr, "        configure {BSB|SBV|SSB|EB|HSB} "
-                     "[--suppress-validation] [global-options] data\n" );
-    fprintf( stderr, "        dump "
-                     "[global-options]\n" );
-    fprintf( stderr, "        dump-eeprom "
-                     "[global-options]\n" );
-    fprintf( stderr, "        dump-user "
-                     "[global-options]\n" );
-    fprintf( stderr, "        erase "
-                     "[--suppress-validation] [global-options]\n" );
-    fprintf( stderr, "        flash "
-                     "[--suppress-validation] [--suppress-bootloader-mem] [global-options] {file|STDIN}\n" );
-    fprintf( stderr, "        flash-eeprom "
-                     "[--suppress-validation] [global-options] {file|STDIN}\n" );
-    fprintf( stderr, "        flash-user "
-                     "[--suppress-validation] [global-options] {file|STDIN}\n" );
-    fprintf( stderr, "        get {bootloader-version|ID1|ID2|BSB|SBV|SSB|EB|\n"
-                     "            manufacturer|family|product-name|\n"
-                     "            product-revision|HSB} "
-                     "[global-options]\n" );
+                     "[--suppress-validation] data\n" );
+    fprintf( stderr, "        dump\n" );
+    fprintf( stderr, "        dump-eeprom\n" );
+    fprintf( stderr, "        dump-user\n" );
+    fprintf( stderr, "        erase [--suppress-validation]\n" );
+    fprintf( stderr, "        flash [--suppress-validation] [--suppress-bootloader-mem]\n"
+                     "                     [--serial=hexdigits:offset] {file|STDIN}\n" );
+    fprintf( stderr, "        flash-eeprom [--suppress-validation]\n"
+                     "                     [--serial=hexdigits:offset] {file|STDIN}\n" );
+    fprintf( stderr, "        flash-user   [--suppress-validation]\n"
+                     "                     [--serial=hexdigits:offset] {file|STDIN}\n" );
+    fprintf( stderr, "        get     {bootloader-version|ID1|ID2|BSB|SBV|SSB|EB|\n"
+                     "                 manufacturer|family|product-name|\n"
+                     "                 product-revision|HSB}\n" );
     fprintf( stderr, "        getfuse {LOCK|EPFL|BOOTPROT|BODLEVEL|BODHYST|\n"
-                     "                BODEN|ISP_BOD_EN|ISP_IO_COND_EN|\n"
-                     "                ISP_FORCE} "
-                     "[global-options]\n" );
+                     "                 BODEN|ISP_BOD_EN|ISP_IO_COND_EN|\n"
+                     "                 ISP_FORCE}\n" );
     fprintf( stderr, "        setfuse {LOCK|EPFL|BOOTPROT|BODLEVEL|BODHYST|\n"
-                     "                BODEN|ISP_BOD_EN|ISP_IO_COND_EN|\n"
-                     "                ISP_FORCE} "
-                     "[global-options] data\n" );
-    fprintf( stderr, "        reset [global-options]\n" );
-    fprintf( stderr, "        start [global-options]\n" );
-    fprintf( stderr, "        version [global-options]\n" );
+                     "                 BODEN|ISP_BOD_EN|ISP_IO_COND_EN|\n"
+                     "                 ISP_FORCE} data\n" );
+    fprintf( stderr, "        setsecure\n" );
+    fprintf( stderr, "        reset\n" );
+    fprintf( stderr, "        start\n" );
 }
 
 static int32_t assign_option( int32_t *arg,
@@ -263,10 +318,31 @@ static int32_t assign_target( struct programmer_arguments *args,
                               struct target_mapping_structure *map )
 {
     while( 0 != *((int32_t *) map) ) {
-        if( 0 == strcasecmp(value, map->name) ) {
+        size_t name_len = strlen(map->name);
+        if( 0 == strncasecmp(value, map->name, name_len)
+            && (value[name_len] == '\0'
+                || value[name_len] == ':')) {
             args->target  = map->value;
             args->chip_id = map->chip_id;
             args->vendor_id = map->vendor_id;
+            args->bus_id = 0;
+            args->device_address = 0;
+            if (value[name_len] == ':') {
+              /* The target name includes USB bus and address info.
+               * This is used to differentiate between multiple dfu 
+               * devices with the same vendor/chip ID numbers. By
+               * specifying the bus and address, mltiple units can
+               * be programmed at one time.
+               */
+              int bus = 0;
+              int address = 0;
+              if( 2 != sscanf(&value[name_len+1], "%i,%i", &bus, &address) )
+                return -1;
+              if (bus <= 0) return -1;
+              if (address <= 0) return -1;
+              args->bus_id = bus;
+              args->device_address = address;
+            }
             args->device_type = map->device_type;
             args->eeprom_memory_size = map->eeprom_memory_size;
             args->flash_page_size = map->flash_page_size;
@@ -290,18 +366,22 @@ static int32_t assign_target( struct programmer_arguments *args,
                 args->flash_address_bottom += map->bootloader_size;
             }
             switch( args->device_type ) {
-                case adc_8051:
+                case ADC_8051:
                     strncpy( args->device_type_string, "8051",
                              DEVICE_TYPE_STRING_MAX_LENGTH );
                     break;
-                case adc_AVR:
+                case ADC_AVR:
                     strncpy( args->device_type_string, "AVR",
                              DEVICE_TYPE_STRING_MAX_LENGTH );
                     break;
-                case adc_AVR32:
+                case ADC_AVR32:
                     strncpy( args->device_type_string, "AVR32",
                              DEVICE_TYPE_STRING_MAX_LENGTH );
                     break;
+                case ADC_XMEGA:
+                    strncpy( args->device_type_string, "XMEGA",
+                             DEVICE_TYPE_STRING_MAX_LENGTH );
+                    break;
             }
             return 0;
         }
@@ -386,6 +466,72 @@ static int32_t assign_global_options( struct programmer_arguments *args,
         }
     }
 
+    /* Find '--serial=<hexdigit+>:<offset>' */
+    for( i = 0; i < argc; i++ ) {
+      if( 0 == strncmp("--serial=", argv[i], 9) ) {
+            *argv[i] = '\0';
+
+            switch( args->command ) {
+                case com_flash:
+                case com_eflash:
+                case com_user: {
+                    char *hexdigits = &argv[i][9];
+                    char *offset_start = hexdigits;
+                    size_t num_digits = 0;
+                    int16_t *serial_data = NULL;
+                    long serial_offset = 0;
+                    size_t j = 0;
+                    char buffer[3] = {0,0,0};
+                    while (*offset_start != ':') {
+                        char c = *offset_start;
+                        if ('\0' == c) {
+                            return -1;
+                        } else if (('0' <= c && c <= '9')
+                            || ('a' <= c && c <= 'f')
+                            || ('A' <= c && c <= 'F')) {
+                            ++offset_start;
+                        } else {
+                          fprintf(stderr, "other character: '%c'\n", *offset_start);
+                            return -1;
+                        }
+                    }
+                    num_digits = offset_start - hexdigits;
+                    if (num_digits & 1) {
+                        fprintf(stderr,"There must be an even number of hexdigits in the serial data\n");
+                        return -1;
+                    }
+                    *offset_start++ = '\0';
+                    if( 1 != sscanf(offset_start, "%ld", &serial_offset) ) {
+                      fprintf(stderr, "sscanf failed\n");
+                      return -1;
+                    }
+                    serial_data = (int16_t *) malloc( (num_digits/2) * sizeof(int16_t) );
+                    for (j=0; j < num_digits; j+=2) {
+                      int data;
+                      buffer[0] = hexdigits[j];
+                      buffer[1] = hexdigits[j+1];
+                      buffer[2] = 0;
+                      if( 1 != sscanf(buffer, "%02x", &data) ) {
+                        fprintf(stderr, "sscanf failed with buffer: %s\n", buffer);
+                        return -1;
+                      }
+                      serial_data[j/2] = (int16_t)data;
+                    }
+                    args->com_flash_data.serial_data = serial_data;
+                    args->com_flash_data.serial_offset = serial_offset;
+                    args->com_flash_data.serial_length = num_digits/2;
+                    break;
+                }
+                default:
+                    /* not supported. */
+                  fprintf(stderr,"command did not match: %d    flash: %d\n", args->command, com_flash);
+                    return -1;
+            }
+            fprintf(stderr, "Success getting serial number\n");
+            break;
+        }
+    }
+
     return 0;
 }
 
@@ -594,6 +740,7 @@ static void print_args( struct programmer_arguments *args )
             break;
         case com_flash:
         case com_eflash:
+        case com_user:
             fprintf( stderr, "   validate: %s\n",
                      (args->com_flash_data.suppress_validation) ?
                         "false" : "true" );
@@ -626,13 +773,30 @@ int32_t parse_arguments( struct programmer_arguments *args,
     args->quiet   = 0;
     args->suppressbootloader = 0;
 
+    /* Special case - check for the help commands which do not require a device type */
+    if( argc == 2 ) {
+        if( 0 == strcasecmp(argv[1], "--version") ) {
+            fprintf( stderr, PACKAGE_STRING "\n");
+            return -1;
+        }
+        if( 0 == strcasecmp(argv[1], "--targets") ) {
+            list_targets();
+            return -1;
+        }
+        if( 0 == strcasecmp(argv[1], "--help") ) {
+            usage();
+            return -1;
+        }
+    }
+
     /* Make sure there are the minimum arguments */
     if( argc < 3 ) {
-        status = -2;
-        goto done;
+        basic_help();
+        return -1;
     }
 
     if( 0 != assign_target(args, argv[1], target_map) ) {
+        fprintf( stderr, "Unsupported target '%s'.\n", argv[1]);
         status = -3;
         goto done;
     }
@@ -681,7 +845,9 @@ done:
         print_args( args );
     }
 
-    if( 0 != status ) {
+    if(-3 == status ) {
+        list_targets();
+    } else if( 0 != status ) {
         usage();
     }
 
