@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: dfu.c 93 2010-02-23 10:17:40Z schmidtw $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,8 +48,10 @@
 #define USB_CLASS_APP_SPECIFIC  0xfe
 #define DFU_SUBCLASS            0x01
 
-/* Wait for 10 seconds before a timeout since erasing/flashing can take some time. */
-#define DFU_TIMEOUT 10000
+/* Wait for 20 seconds before a timeout since erasing/flashing can take some time.
+ * The longest erase cycle is for the AT32UC3A0512-TA automotive part,
+ * which needs a timeout of at least 19 seconds to erase the whole flash. */
+#define DFU_TIMEOUT 20000
 
 /* Time (in ms) for the device to wait for the usb reset after being told to detach
  * before the giving up going into dfu mode. */
@@ -369,6 +371,8 @@ int32_t dfu_abort( dfu_device_t *device )
 #ifdef HAVE_LIBUSB_1_0
 struct libusb_device *dfu_device_init( const uint32_t vendor,
                                        const uint32_t product,
+                                       const uint32_t bus_number,
+                                       const uint32_t device_address,
                                        dfu_device_t *dfu_device,
                                        const dfu_bool initial_abort,
                                        const dfu_bool honor_interfaceclass )
@@ -400,9 +404,13 @@ retry:
                 descriptor.idVendor, descriptor.idProduct );
 
         if( (vendor  == descriptor.idVendor) &&
-            (product == descriptor.idProduct) )
+            (product == descriptor.idProduct) &&
+            ((bus_number == 0)
+             || ((libusb_get_bus_number(device) == bus_number) &&
+                 (libusb_get_device_address(device) == device_address))) )
         {
             int32_t tmp;
+            DEBUG( "found device at USB:%d,%d\n", libusb_get_bus_number(device), libusb_get_device_address(device) );
             /* We found a device that looks like it matches...
              * let's try to find the DFU interface, open the device
              * and claim it. */
@@ -457,6 +465,8 @@ retry:
 #else
 struct usb_device *dfu_device_init( const uint32_t vendor,
                                     const uint32_t product,
+                                    const uint32_t bus_number,
+                                    const uint32_t device_address,
                                     dfu_device_t *dfu_device,
                                     const dfu_bool initial_abort,
                                     const dfu_bool honor_interfaceclass )
@@ -479,9 +489,13 @@ retry:
         for( usb_bus = usb_get_busses(); NULL != usb_bus; usb_bus = usb_bus->next ) {
             for( device = usb_bus->devices; NULL != device; device = device->next) {
                 if(    (vendor  == device->descriptor.idVendor)
-                    && (product == device->descriptor.idProduct) )
+                    && (product == device->descriptor.idProduct)
+                    && ((bus_number == 0)
+                        || (device->devnum == device_address
+                            && (usb_bus->location >> 24) == bus_number)))
                 {
                     int32_t tmp;
+                    DEBUG( "found device at USB:%d,%d\n", device->devnum, (usb_bus->location >> 24) );
                     /* We found a device that looks like it matches...
                      * let's try to find the DFU interface, open the device
                      * and claim it. */
@@ -924,6 +938,9 @@ static void dfu_msg_response_output( const char *function, const int32_t result
             case -ENODEV:
                 msg = "-ENODEV: Device was removed";
                 break;
+            case -EIO:
+                msg = "-EIO: USB I/O error";
+                break;
 #ifdef EREMOTEIO
             case -EREMOTEIO:
                 msg = "-EREMOTEIO: Short packet detected";
